@@ -110,6 +110,53 @@ crew_class_controller <- R6::R6Class(
     .autoscaling = FALSE,
     .queue = NULL,
     .resolved = -1L,
+    .register_started = function() {
+      private$.tasks <- new.env(parent = emptyenv(), hash = TRUE)
+      private$.pushed <- 0L
+      private$.popped <- 0L
+      private$.crash_log <- new.env(parent = emptyenv(), hash = TRUE)
+      private$.summary <- list(
+        controller = private$.launcher$name,
+        tasks = 0L,
+        seconds = 0,
+        success = 0L,
+        error = 0L,
+        crash = 0L,
+        cancel = 0L,
+        warning = 0L
+      )
+      private$.backlog <- character(0L)
+      private$.queue <- crew_queue()
+      private$.resolved <- -1L
+    },
+    .name_new_task = function(name) {
+      tasks <- .subset2(private, ".tasks")
+      if (is.null(name)) {
+        name <- name_task_tempfile()
+        name <- if_any(
+          is.null(.subset2(tasks, name)),
+          name,
+          name_task_nanonext()
+        )
+      }
+      if (!is.null(.subset2(tasks, name))) {
+        crew_error(
+          message = paste(
+            "crew task name",
+            name,
+            "already found in the task list. Before pushing a task",
+            "of the same name, please wait for it to resolve, then",
+            "use pop() or collect() to remove it from the controller."
+          )
+        )
+      }
+      name
+    },
+    .push_task = function(name, task) {
+      tasks <- .subset2(private, ".tasks")
+      tasks[[name]] <- task
+      private$.pushed <- .subset2(private, ".pushed") + 1L
+    },
     .resolve = function(force) {
       queue <- .subset2(private, ".queue")
       if ((!force) && .subset2(queue, "nonempty")()) {
@@ -180,7 +227,7 @@ crew_class_controller <- R6::R6Class(
     }
   ),
   active = list(
-    #' @field client Router object.
+    #' @field client Client object.
     client = function() {
       .subset2(private, ".client")
     },
@@ -231,7 +278,7 @@ crew_class_controller <- R6::R6Class(
   public = list(
     #' @description `mirai` controller constructor.
     #' @return An `R6` controller object.
-    #' @param client Router object. See [crew_controller()].
+    #' @param client Client object. See [crew_controller()].
     #' @param launcher Launcher object. See [crew_controller()].
     #' @param crashes_max See [crew_controller()].
     #' @param backup See [crew_controller()].
@@ -408,23 +455,7 @@ crew_class_controller <- R6::R6Class(
           url = private$.client$url,
           profile = private$.client$profile
         )
-        private$.tasks <- new.env(parent = emptyenv(), hash = TRUE)
-        private$.pushed <- 0L
-        private$.popped <- 0L
-        private$.crash_log <- new.env(parent = emptyenv(), hash = TRUE)
-        private$.summary <- list(
-          controller = private$.launcher$name,
-          tasks = 0L,
-          seconds = 0,
-          success = 0L,
-          error = 0L,
-          crash = 0L,
-          cancel = 0L,
-          warning = 0L
-        )
-        private$.backlog <- character(0L)
-        private$.queue <- crew_queue()
-        private$.resolved <- -1L
+        private$.register_started()
       }
       invisible()
     },
@@ -603,6 +634,7 @@ crew_class_controller <- R6::R6Class(
       controller = NULL
     ) {
       .subset2(self, "start")()
+      name <- private$.name_new_task(name)
       if (substitute) {
         command <- substitute(command)
       }
@@ -611,26 +643,6 @@ crew_class_controller <- R6::R6Class(
       } else {
         .timeout <- seconds_timeout * 1000
       }
-      tasks <- .subset2(private, ".tasks")
-      if (is.null(name)) {
-        name <- name_task_tempfile()
-        name <- if_any(
-          is.null(.subset2(tasks, name)),
-          name,
-          name_task_nanonext()
-        )
-      }
-      if (!is.null(.subset2(tasks, name))) {
-        crew_error(
-          message = paste(
-            "crew task name",
-            name,
-            "already found in the task list. Before pushing a task",
-            "of the same name, please wait for it to resolve, then",
-            "use pop() or collect() to remove it from the controller."
-          )
-        )
-      }
       backup <- .subset2(private, ".backup")
       if (!is.null(backup)) {
         max <- .subset2(private, ".crashes_max")
@@ -669,8 +681,7 @@ crew_class_controller <- R6::R6Class(
         .timeout = .timeout,
         .compute = .subset2(.subset2(private, ".client"), "profile")
       )
-      tasks[[name]] <- task
-      private$.pushed <- .subset2(self, "pushed") + 1L
+      .subset2(private, ".push_task")(name, task)
       if (scale) {
         .subset2(self, "scale")(throttle = throttle)
       }
@@ -1624,6 +1635,7 @@ crew_class_controller <- R6::R6Class(
       out
     },
     #' @description Cancel one or more tasks.
+    #' @return `NULL` (invisibly).
     #' @param names Character vector of names of tasks to cancel.
     #'   Those names must have been manually supplied by `push()`.
     #' @param all `TRUE` to cancel all tasks, `FALSE` otherwise.
