@@ -1,5 +1,5 @@
 name_worker <- function(launcher, worker) {
-  paste("crew", launcher, worker, sep = "-")
+  paste("crew-worker", launcher, worker, sep = "-")
 }
 
 name_task_tempfile <- function() {
