@@ -18,14 +18,15 @@ crew_test("relay and waiting", {
   expect_silent(x$validate())
   nanonext::cv_signal(x$from)
   nanonext::msleep(250)
+  throttle <- crew_throttle()
   expect_equal(nanonext::cv_value(x$from), 1L)
   expect_equal(nanonext::cv_value(x$condition), 1L)
   expect_equal(nanonext::cv_value(x$to), 1L)
-  expect_true(x$wait())
+  expect_true(x$wait(throttle = throttle))
   expect_equal(nanonext::cv_value(x$from), 1L)
   expect_equal(nanonext::cv_value(x$condition), 0L)
   expect_equal(nanonext::cv_value(x$to), 1L)
-  expect_false(x$wait(seconds_timeout = 0))
+  expect_false(x$wait(throttle = throttle))
   expect_equal(nanonext::cv_value(x$from), 1L)
   expect_equal(nanonext::cv_value(x$condition), 0L)
   expect_equal(nanonext::cv_value(x$to), 1L)
